@@ -169,6 +169,7 @@ sub remove {
     my $statement = sprintf "DELETE FROM bulkmailer_table WHERE packetid_bulkmailer = %s AND messagekey_bulkmailer = %s",$dbh->quote($packetid),$dbh->quote($messagekey),;
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }	   
     return ($dbh->do($statement));
 }
@@ -181,6 +182,7 @@ sub messageasstring {
     
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
     unless ($sth = $dbh->prepare($statement)) {
 	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
@@ -214,6 +216,7 @@ sub message_from_spool {
 
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
     unless ($sth = $dbh->prepare($statement)) {
 	do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
@@ -269,8 +272,8 @@ sub merge_msg {
     }
     ## Get the Content-Type / Charset / Content-Transfer-encoding of a message
     my $type      = $entity->mime_type;
-    my $charset   = unmime $entity->head->mime_attr('content-type.charset');
-    my $encoding  = unmime $entity->head->mime_encoding;
+    my $charset   = &MIME::WordDecoder::unmime($entity->head->mime_attr('content-type.charset'));
+    my $encoding  = &MIME::WordDecoder::unmime($entity->head->mime_encoding);
 
     my $message_output;
     my $IO;
@@ -280,10 +283,16 @@ sub merge_msg {
 	
 	if(defined $body){
 	    ## --------- Initial Charset to UTF-8 --------- ##
-	    unless($charset =~ /UTF-8/){
-		# Put the charset to UTF-8
-		Encode::from_to($body, $charset, 'UTF-8');
-	      }
+	    ## We use find_encoding() to ensure that's a valid charset
+	    if ($charset && ref Encode::find_encoding($charset)) { 
+		unless($charset =~ /UTF-8/){
+		    # Put the charset to UTF-8
+		    Encode::from_to($body, $charset, 'UTF-8');
+		  }       
+	    }else {
+		&do_log('err', "Incorrect charset '%s' ; cannot encode in this charset", $charset);
+	    }
+
 	    ## PARSAGE ##
 	    
 	    &merge_data('rcpt' => $rcpt,
@@ -296,11 +305,16 @@ sub merge_msg {
 			); 
 	    $body = $message_output;
 
-	    unless($charset =~ /UTF-8/){
-		# Put the charset to the initial
-		Encode::from_to($body, 'UTF-8',$charset);
-	      }
-	    
+	    ## We use find_encoding() to ensure that's a valid charset
+	    if ($charset && ref Encode::find_encoding($charset)) { 
+		unless($charset =~ /UTF-8/){
+		    # Put the charset to UTF-8
+			Encode::from_to($body, 'UTF-8',$charset);
+		  }       
+	    }else {
+		&do_log('err', "Incorrect charset '%s' ; cannot encode in this charset", $charset);
+	    }
+
 	    # Write the new body in the entity
 	    unless($IO = $entity->bodyhandle->open("w") || die "open body: $!"){
 		&do_log('err', "Can't open Entity");
@@ -421,6 +435,7 @@ sub store {
     
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
     
     $msg = MIME::Base64::encode($msg);
@@ -546,6 +561,7 @@ sub purge_bulkspool {
 
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
     my $statement = "SELECT messagekey_bulkspool AS messagekey FROM bulkspool_table LEFT JOIN bulkmailer_table ON messagekey_bulkspool = messagekey_bulkmailer WHERE messagekey_bulkmailer IS NULL AND lock_bulkspool = 0";
     unless ($sth = $dbh->prepare($statement)) {
@@ -581,6 +597,7 @@ sub remove_bulkspool_message {
 
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
 
     my $statement = sprintf "DELETE FROM %s WHERE %s = '%s'",$table,$key,$messagekey;
@@ -626,6 +643,7 @@ sub store_test {
 
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
     
     $priority_message = 9;
@@ -677,6 +695,7 @@ sub get_remaining_packets_count {
 
     unless ($dbh and $dbh->ping) {
 	return undef unless &List::db_connect();
+	$dbh = &List::db_get_handler();
     }
 
     my $statement = "SELECT COUNT(*) FROM bulkmailer_table";
