@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 6132 $ ; $Date: 2009-08-21 14:17:56 +0200 (ven 21 aoû 2009) $ 
+# RCS Identication ; $Revision: 7334 $ ; $Date: 2012-04-24 15:12:12 +0200 (mar 24 avr 2012) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -57,9 +57,9 @@ my %params =
 my %valid_robot_key_words;
 my %optional_key_words;
 foreach my $hash(@confdef::params){
-    $valid_robot_key_words{$hash->{'name'}} = 1 if ($hash->{'vhost'});
+    $valid_robot_key_words{$hash->{'name'}} = 1 if ($hash->{'vhost'});    
     $valid_robot_key_words{$hash->{'name'}} = 'db' if (defined($hash->{'db'}) and $hash->{'db'} ne 'none');
-    $optional_key_words{$hash->{'name'}} = 1 if ($hash->{'optional'});
+    $optional_key_words{$hash->{'name'}} = 1 if ($hash->{'optional'}); 
 }
 
 my %old_params = (
@@ -99,9 +99,15 @@ my $wwsconf;
 our %Conf = ();
 
 ## Loads and parses the configuration file. Reports errors if any.
+# do not try to load database values if $no_db is set ;
+# do not change gloval hash %Conf if $return_result  is set ;
+# we known that's dirty, this proc should be rewritten without this global var %Conf
 sub load {
     my $config = shift;
     my $no_db = shift;
+    my $return_result = shift;
+
+
     my $line_num = 0;
     my $config_err = 0;
     my($i, %o);
@@ -115,30 +121,31 @@ sub load {
         $line_num++;
         # skip empty or commented lines
         next if (/^\s*$/ || /^[#;]/);
-        # match "keyword value" pattern
-        if (/^(\S+)\s+(.+)$/) {
-            my ($keyword, $value) = ($1, $2);
-            $value =~ s/\s*$//;
-            ##  'tri' is a synonyme for 'sort'
-            ## (for compatibily with old versions)
-            $keyword = 'sort' if ($keyword eq 'tri');
-            ##  'key_password' is a synonyme for 'key_passwd'
-            ## (for compatibily with old versions)
-            $keyword = 'key_passwd' if ($keyword eq 'key_password');
-            ## Special case: `command`
-            if ($value =~ /^\`(.*)\`$/) {
-                $value = qx/$1/;
-                chomp($value);
-            }
-            $o{$keyword} = [ $value, $line_num ];
-        } else {
-            printf STDERR
-                gettext("Error at line %d: %s\n"), $line_num, $config, $_;
-            $config_err++;
-        }
+	    # match "keyword value" pattern
+	    if (/^(\S+)\s+(.+)$/) {
+		my ($keyword, $value) = ($1, $2);
+		$value =~ s/\s*$//;
+		##  'tri' is a synonym for 'sort'
+		## (for compatibilyty with older versions)
+		$keyword = 'sort' if ($keyword eq 'tri');
+		##  'key_password' is a synonym for 'key_passwd'
+		## (for compatibilyty with older versions)
+		$keyword = 'key_passwd' if ($keyword eq 'key_password');
+		## Special case: `command`
+		if ($value =~ /^\`(.*)\`$/) {
+		    $value = qx/$1/;
+		    chomp($value);
+		}
+		$o{$keyword} = [ $value, $line_num ];
+	    } else {
+		printf STDERR  gettext("Error at line %d: %s\n"), $line_num, $config, $_;
+		$config_err++;
+	    }
     }
     close(IN);
 
+    return (\%o) if ($return_result);
+
     ## Hardcoded values
     foreach my $p (keys %hardcoded_params) {
 	$Ignored_Conf{$p} = $o{$p}[0] if (defined $o{$p});
@@ -232,13 +239,23 @@ sub load {
     if ($Conf{'lock_method'} eq 'nfs') {
         eval "require File::NFSLock";
         if ($@) {
-            &do_log(
-                'err',
-                "Failed to load File::NFSLock perl module ; setting 'lock_method' to 'flock'"
-            );
+            &do_log('err',"Failed to load File::NFSLock perl module ; setting 'lock_method' to 'flock'" );
             $Conf{'lock_method'} = 'flock';
         }
     }
+		 
+    ## Some parameters require CPAN modules
+    if ($Conf{'DKIM_feature'} eq 'on') {
+        eval "require Mail::DKIM";
+        if ($@) {
+            &do_log('err', "Failed to load Mail::DKIM perl module ; setting 'DKIM_feature' to 'off'");
+            $Conf{'DKIM_feature'} = 'off';
+        }
+    }
+    unless ($Conf{'DKIM_feature'} eq 'on'){
+	# dkim_signature_apply_ on nothing if DKIM_feature is off
+	$Conf{'dkim_signature_apply_on'} = ['']; # empty array
+    }
 
     ## Load charset.conf file if necessary.
     if($Conf{'legacy_character_support_feature'} eq 'on'){
@@ -349,9 +366,12 @@ sub load {
     $Conf{'pictures_url'}  = $Conf{'static_content_url'}.'/pictures/';
     $Conf{'pictures_path'}  = $Conf{'static_content_path'}.'/pictures/';
 	
-
-
-
+    ## Parsing custom robot parameters.
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+	if($Conf{'robots'}{$robot}{'custom_robot_parameter'} =~ /(\S+)\s*\;\s*(.+)/) {
+	    $Conf{'robots'}{$robot}{'custom_robot_parameter'} = {$1 => $2};
+	}
+    }
     return 1;
 }    
 
@@ -424,6 +444,41 @@ sub load_nrcpt_by_domain {
   return ($nrcpt_by_domain);
 }
 
+#load a confif file without any default and inherited property 
+sub load_conf_file {
+
+    my $config_type = shift; #  'robot' or other
+    my $path=shift;
+
+    do_log('info',"load_config_file($config_type,$path)");
+
+    my %thisconf;
+
+    return undef unless (-f $path) ;
+    return undef unless (-r $path) ;
+    return undef unless (open (CONF,$path));
+    while (<CONF>) {
+	next if (/^\s*$/o || /^[\#\;]/o);
+	if (/^\s*(\S+)\s+(.+)\s*$/io) {
+	    my($keyword, $value) = ($1, $2);
+	    $value =~ s/\s*$//;
+	    $keyword = lc($keyword);
+	    
+	    ## Not all parameters should be lowercased
+	    ## We should define which parameter needs to be lowercased
+	    #$value = lc($value) unless ($keyword eq 'title' || $keyword eq 'logo_html_definition' || $keyword eq 'lang');
+
+	    if ($config_type eq 'robot') {
+		unless($valid_robot_key_words{$keyword}) {
+		    printf STDERR "load_config_file robot config: unknown keyword $keyword\n";
+		    next;
+		}
+	    }# elseif(some other config type) { some other check of valid keywords
+	    $thisconf{$keyword} = $value;
+	}
+    }
+    return (\%thisconf);
+}
 
 ## load each virtual robots configuration files
 sub load_robots {
@@ -440,7 +495,7 @@ sub load_robots {
 	printf STDERR "Unable to open directory $Conf{'etc'} for virtual robots config\n" ;
 	return undef;
     }
-    my $exiting = 0 ;
+    my $exiting = 0;
     ## Set the defaults based on sympa.conf and wwsympa.conf first
     foreach my $key (keys %valid_robot_key_words) {
 	if(defined $wwsconf->{$key}){
@@ -449,7 +504,7 @@ sub load_robots {
 	    $robot_conf->{$Conf{'domain'}}{$key} = $Conf{$key};
 	}else{
 	    unless ($optional_key_words{$key}){
-		printf STDERR "Parameter $key seems to be neither a wwsympa.conf nor a sympa.conf files.\n" ;
+		printf STDERR "Parameter $key seems to be neither a wwsympa.conf nor a sympa.conf parameter.\n" ;
 		$exiting = 1;
 	    }
 	}
@@ -471,7 +526,7 @@ sub load_robots {
 	    printf STDERR "load robots config: Unable to open $Conf{'etc'}/$robot/robot.conf\n"; 
 	    next ;
 	}
-	
+
 	while (<ROBOT_CONF>) {
 	    next if (/^\s*$/o || /^[\#\;]/o);
 	    if (/^\s*(\S+)\s+(.+)\s*$/io) {
@@ -494,6 +549,7 @@ sub load_robots {
 	}
 	# listmaster is a list of email separated by commas
 	$robot_conf->{$robot}{'listmaster'} =~ s/\s//g;
+
 	@{$robot_conf->{$robot}{'listmasters'}} = split(/,/, $robot_conf->{$robot}{'listmaster'})
 	    if $robot_conf->{$robot}{'listmaster'};
 
@@ -691,7 +747,7 @@ sub checkfiles_as_root {
   my $config_err = 0;
 
     ## Check aliases file
-    unless (-f $Conf{'sendmail_aliases'}) {
+    unless (-f $Conf{'sendmail_aliases'} || ($Conf{'sendmail_aliases'} =~ /^none$/i)) {
 	unless (open ALIASES, ">$Conf{'sendmail_aliases'}") {
 	    &do_log('err',"Failed to create aliases file %s", $Conf{'sendmail_aliases'});
 	    # printf STDERR "Failed to create aliases file %s", $Conf{'sendmail_aliases'};
@@ -787,18 +843,34 @@ sub checkfiles {
 		do_log('err', 'Unable to create spool %s', $Conf{$qdir});
 		$config_err++;
 	    }
+            unless (&tools::set_file_rights(
+                    file  => $Conf{$qdir},
+                    user  => Sympa::Constants::USER,
+                    group => Sympa::Constants::GROUP,
+            )) {
+                &do_log('err','Unable to set rights on %s',$Conf{$qdir});
+		$config_err++;
+            }
 	}
     }
 
     ## Also create associated bad/ spools
-    foreach my $qdir ('queue','queuedistribute','queueautomatic')
-    {
-	unless (-d $Conf{$qdir}.'/bad') {
-	    do_log('info', "creating spool $Conf{$qdir}/bad");
-	    unless ( mkdir ($Conf{$qdir}.'/bad', 0775)) {
-		do_log('err', 'Unable to create spool %s', $Conf{$qdir}.'/bad');
+    foreach my $qdir ('queue','queuedistribute','queueautomatic') {
+        my $subdir = $Conf{$qdir}.'/bad';
+	unless (-d $subdir) {
+	    do_log('info', "creating spool $subdir");
+	    unless ( mkdir ($subdir, 0775)) {
+		do_log('err', 'Unable to create spool %s', $subdir);
 		$config_err++;
 	    }
+            unless (&tools::set_file_rights(
+                    file  => $subdir,
+                    user  => Sympa::Constants::USER,
+                    group => Sympa::Constants::GROUP,
+            )) {
+                &do_log('err','Unable to set rights on %s',$subdir);
+		$config_err++;
+            }
 	}
     }
 
@@ -900,11 +972,12 @@ sub checkfiles {
 	foreach my $css ('style.css','print.css','fullPage.css','print-preview.css') {
 
 	    $param->{'css'} = $css;
-
+	    my $css_tt2_path = &tools::get_filename('etc',{}, 'web_tt2/css.tt2', $robot, undef);
+	    
 	    ## Update the CSS if it is missing or if a new css.tt2 was installed
 	    if (! -f $dir.'/'.$css ||
-		(stat(Sympa::Constants::DEFAULTDIR . '/web_tt2/css.tt2'))[9] > (stat($dir.'/'.$css))[9]) {
-		&do_log('notice',"Updating static CSS file $dir/$css ; previous file renamed");
+		(stat($css_tt2_path))[9] > (stat($dir.'/'.$css))[9]) {
+		&do_log('notice',"TT2 file $css_tt2_path has changed; updating static CSS file $dir/$css ; previous file renamed");
 		
 		## Keep copy of previous file
 		rename $dir.'/'.$css, $dir.'/'.$css.'.'.time;
@@ -1036,7 +1109,7 @@ sub _load_auth {
 					    'ldap_ssl_ciphers' => '[\w:]+',
 					    'force_email_verify' => '1',
 					    'internal_email_by_netid' => '1',
-					    'netid_http_header' => '\w+',
+					    'netid_http_header' => '[\w\-\.]+',
 					},
 			  'authentication_info_url' => 'http(s)?:/.*'
 			  );
@@ -1130,6 +1203,13 @@ sub _load_auth {
 		  $Conf{'generic_sso_id'}{$robot}{$current_paragraph->{'service_id'}} =  $#paragraphs+1 ; 
 		  $current_paragraph->{'ldap_scope'} ||= 'sub'; ## Force the default scope because '' is interpreted as 'base'
 		  $current_paragraph->{'http_header_value_separator'} ||= ';'; ## default value for http_header_value_separator is ';'
+
+		  ## CGI.pm changes environment variable names ('-' => '_')
+		  ## declared environment variable names needs to be transformed accordingly
+		  foreach my $parameter ('http_header_list','email_http_header','netid_http_header') {
+		      $current_paragraph->{$parameter} =~ s/\-/\_/g if (defined $current_paragraph->{$parameter});
+		  }
+
 		}elsif($current_paragraph->{'auth_type'} eq 'ldap') {
 		    $Conf{'ldap'}{$robot}  ++ ;
 		    $Conf{'use_passwd'}{$robot} = 1;
@@ -1368,7 +1448,8 @@ sub load_generic_conf_file {
 		}
 		
 		unless ($paragraph[$i] =~ /^\s*$key\s+($structure{$pname}{'format'}{$key}{'format'})\s*$/i) {
-		    printf STDERR 'Bad entry "%s" in paragraph "%s" in %s\n', $paragraph[$i], $key, $pname, $config_file;
+		    chomp($paragraph[$i]);
+		    printf STDERR 'Bad entry "%s" for key "%s", paragraph "%s" in file "%s"\n', $paragraph[$i], $key, $pname, $config_file;
 		    return undef if $on_error eq 'abort';
 		    next;
 		}
@@ -1417,6 +1498,7 @@ sub load_generic_conf_file {
 	    }
 
 	    unless ($paragraph[0] =~ /^\s*$pname\s+($structure{$pname}{'format'})\s*$/i) {
+		chomp($paragraph[0]);
 		printf STDERR 'Bad entry "%s" in %s\n', $paragraph[0], $config_file ;
 		return undef if $on_error eq 'abort';
 		next;
@@ -1472,5 +1554,52 @@ sub _load_a_param {
     }
 }
 
+# Store configs to database
+sub conf_2_db {
+    my $config_file = shift;
+    do_log('info',"conf_2_db");
+
+    my @conf_parameters = @confdef::params ;
+
+    # store in database robots parameters.
+    my $robots_conf = &load_robots ; #load only parameters that are in a robot.txt file (do not apply defaults). 
+
+    unless (opendir DIR,$Conf{'etc'} ) {
+	printf STDERR "Unable to open directory $Conf{'etc'} for virtual robots config\n" ;
+	return undef;
+    }
+
+    foreach my $robot (readdir(DIR)) {
+	next unless (-d "$Conf{'etc'}/$robot");
+	next unless (-f "$Conf{'etc'}/$robot/robot.conf");
+	
+	my $config = &load_conf_file('robot',$Conf{'etc'}.'/'.$robot.'/robot.txt');
+	
+	for my $i ( 0 .. $#conf_parameters ) {
+	    if ($conf_parameters[$i]->{'name'}) { #skip separators in conf_parameters structure
+		if (($conf_parameters[$i]->{'vhost'} eq '1') && #skip parameters that can't be define by robot so not to be loaded in db at that stage 
+		    ($config->{$conf_parameters[$i]->{'name'}})){
+		    &Conf::set_robot_conf($robot, $conf_parameters[$i]->{'name'}, $config->{$conf_parameters[$i]->{'name'}});
+		}
+	    }
+	}
+    }
+    closedir (DIR);
+
+    # store in database sympa;conf and wwsympa.conf
+    
+    ## Load configuration file. Ignoring database config and get result
+    my $global_conf;
+    unless ($global_conf= Conf::load($config_file,1,'return_result')) {
+	&fatal_err("Configuration file $config_file has errors.");  
+    }
+    
+    for my $i ( 0 .. $#conf_parameters ) {
+	if (($conf_parameters[$i]->{'edit'} eq '1') && $global_conf->{$conf_parameters[$i]->{'name'}}) {
+	    &Conf::set_robot_conf("*",$conf_parameters[$i]->{'name'},$global_conf->{$conf_parameters[$i]->{'name'}}[0]);
+	}       
+    }
+}
+
 ## Packages must return true.
 1;
