@@ -1,5 +1,5 @@
 # Message.pm - This module includes Message processing functions
-#<!-- RCS Identication ; $Revision: 6702 $ ; $Date: 2010-10-01 16:02:48 +0200 (ven 01 oct 2010) $ --> 
+#<!-- RCS Identication ; $Revision: 7339 $ ; $Date: 2012-05-09 06:27:57 +0200 (mer 09 mai 2012) $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
@@ -173,23 +173,41 @@ sub new {
 	do_log('err', "Invalid From: field '%s'", $message->{'sender'});
 	return undef;
     }
-    ## Store decoded subject
-    my $subject = $hdr->get('Subject');
-    my @decoded_subject = MIME::EncWords::decode_mimewords($subject);
-    foreach my $token (@decoded_subject) {
-	$message->{'subject_charset'} ||= $token->[1];
-    }
-    
-    ## Don't try to decode if subject is empty to prevent a bug of 
-    ## decode_mimeords that would set the subject to 'Charset'
-    ## Strangely the problem disappeared when using $subject instead of $hdr->get('Subject')
-    ## as parameter to deocde_mimewords()
-    unless ($subject =~ /^$/) {
-	$message->{'decoded_subject'} = MIME::EncWords::decode_mimewords(
-									 $subject, Charset=>'utf8');
-	chomp $message->{'decoded_subject'};
-    }
 
+    ## Store decoded subject and its original charset
+    my $subject = $hdr->get('Subject');
+    if ($subject =~ /\S/) {
+	my @decoded_subject = MIME::EncWords::decode_mimewords($subject);
+	$message->{'subject_charset'} = 'US-ASCII';
+	foreach my $token (@decoded_subject) {
+	    unless ($token->[1]) {
+		# don't decode header including raw 8-bit bytes.
+		if ($token->[0] =~ /[^\x00-\x7F]/) {
+		    $message->{'subject_charset'} = undef;
+		    last;
+		}
+		next;
+	    }
+	    my $cset = MIME::Charset->new($token->[1]);
+	    # don't decode header encoded with unknown charset.
+	    unless ($cset->decoder) {
+		$message->{'subject_charset'} = undef;
+		last;
+	    }
+	    unless ($cset->output_charset eq 'US-ASCII') {
+		$message->{'subject_charset'} = $token->[1];
+	    }
+	}
+    } else {
+	$message->{'subject_charset'} = undef;
+    }
+    if ($message->{'subject_charset'}) {
+	$message->{'decoded_subject'} =
+	    MIME::EncWords::decode_mimewords($subject, Charset => 'utf8');
+    } else {
+	$message->{'decoded_subject'} = $subject;
+    }
+    chomp $message->{'decoded_subject'};
 
     ## Extract recepient address (X-Sympa-To)
     $message->{'rcpt'} = $hdr->get('X-Sympa-To');
@@ -479,11 +497,28 @@ sub fix_html_part {
 	    push @newparts, &fix_html_part($_,$robot);
 	}
 	$part->parts(\@newparts);
-    } elsif (MIME::Tools::textual_type($eff_type)) {
+    } elsif ($eff_type =~ /^text\/html/i) {
 	my $bodyh = $part->bodyhandle;
 	# Encoded body or null body won't be modified.
 	return $part if !$bodyh or $bodyh->is_encoded;
-	my $filtered_body = &tools::sanitize_html('string' => $bodyh->as_string, 'robot'=> $robot);
+
+	my $body = $bodyh->as_string;
+	# Re-encode parts with 7-bit charset (ISO-2022-*), since
+	# StripScripts cannot handle them correctly.
+	my $cset = MIME::Charset->new($part->head->mime_attr('Content-Type.Charset') || '');
+	unless ($cset->decoder) {
+	    # Charset is unknown.  Detect 7-bit charset.
+	    my ($dummy, $charset) =
+		$cset->body_encode($body, Detect7Bit => 'YES');
+	    $cset = MIME::Charset->new($charset);
+	}
+	if ($cset->decoder and $cset->as_string =~ /^ISO-2022-/i) {
+	    $part->head->mime_attr('Content-Type.Charset', 'UTF-8');
+	    $cset->encoder('UTF-8');
+	    $body = $cset->encode($body);
+	}
+
+	my $filtered_body = &tools::sanitize_html('string' => $body, 'robot'=> $robot);
 
 	my $io = $bodyh->open("w");
 	unless (defined $io) {
