@@ -26,26 +26,26 @@ use Exporter;
 our @ISA = qw(Exporter);
 
 
-use constant VERSION => '6.1.7';
+use constant VERSION => '6.1.11';
 use constant USER    => 'sympa';
 use constant GROUP   => 'sympa';
 
-use constant CONFIG           => '/tmp/sympapipo/etc/sympa.conf';
-use constant WWSCONFIG        => '/tmp/sympapipo/etc/wwsympa.conf';
-use constant SENDMAIL_ALIASES => '/etc/mail/aliases-sympapipo';
+use constant CONFIG           => '/usr/local/sympa-stable/etc/sympa.conf';
+use constant WWSCONFIG        => '/usr/local/sympa-stable/etc/wwsympa.conf';
+use constant SENDMAIL_ALIASES => '/etc/mail/aliases-sympa-stable';
 
-use constant PIDDIR     => '/tmp/sympapipo';
-use constant EXPLDIR    => '/tmp/sympapipo/list_data';
-use constant SPOOLDIR   => '/tmp/sympapipo/spool';
-use constant SYSCONFDIR => '/tmp/sympapipo/etc';
-use constant LOCALEDIR  => '/tmp/sympapipo/locale';
-use constant LIBEXECDIR => '/tmp/sympapipo/bin';
-use constant SBINDIR    => '/tmp/sympapipo/bin';
-use constant SCRIPTDIR  => '/tmp/sympapipo/bin';
-use constant MODULEDIR  => '/tmp/sympapipo/bin';
-use constant DEFAULTDIR => '/tmp/sympapipo/default';
-use constant STATICDIR  => '/tmp/sympapipo/static_content';
-use constant ARCDIR    => '/tmp/sympapipo/arc';
-use constant BOUNCEDIR  => '/tmp/sympapipo/bounce';
+use constant PIDDIR     => '/usr/local/sympa-stable';
+use constant EXPLDIR    => '/usr/local/sympa-stable/list_data';
+use constant SPOOLDIR   => '/usr/local/sympa-stable/spool';
+use constant SYSCONFDIR => '/usr/local/sympa-stable/etc';
+use constant LOCALEDIR  => '/usr/local/sympa-stable/locale';
+use constant LIBEXECDIR => '/usr/local/sympa-stable/bin';
+use constant SBINDIR    => '/usr/local/sympa-stable/bin';
+use constant SCRIPTDIR  => '/usr/local/sympa-stable/bin';
+use constant MODULEDIR  => '/usr/local/sympa-stable/bin';
+use constant DEFAULTDIR => '/usr/local/sympa-stable/default';
+use constant STATICDIR  => '/usr/local/sympa-stable/static_content';
+use constant ARCDIR    => '/usr/local/sympa-stable/arc';
+use constant BOUNCEDIR  => '/usr/local/sympa-stable/bounce';
 
 1;
