@@ -1,5 +1,5 @@
 # Upgrade.pm - This module gathers all subroutines used to upgrade Sympa data structures
-#<!-- RCS Identication ; $Revision: 7055 $ --> 
+#<!-- RCS Identication ; $Revision: 7356 $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
@@ -30,6 +30,7 @@ use POSIX qw(strftime);
 use Conf;
 use Log;
 use Sympa::Constants;
+use Data::Dumper;
 
 ## Return the previous Sympa version, ie the one listed in data_structure.version
 sub get_previous_version {
@@ -702,6 +703,52 @@ sub upgrade {
 		}
 		
    }	
+   if (&tools::lower_version($previous_version, '6.1.11')) {
+       ## Exclusion table was not robot-enabled.
+       &Log::do_log('notice','fixing robot column of exclusion table.');
+	my $statement = "SELECT * FROM exclusion_table"; 
+	my $sth;
+	unless ($sth = $dbh->prepare($statement)) {
+	    &Log::do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
+	}
+	unless ($sth->execute) {
+	    &Log::do_log('err','Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
+	}
+	my @robots = &List::get_robots();
+	while (my $data = $sth->fetchrow_hashref){
+	    next if (defined $data->{'robot_exclusion'} && $data->{'robot_exclusion'} ne '');
+	    ## Guessing right robot for each exclusion.
+	    my $valid_robot = '';
+	    my @valid_robot_candidates;
+	    foreach my $robot (@robots) {
+		if (my $list = new List($data->{'list_exclusion'},$robot)) {
+		    if ($list->is_user($data->{'user_exclusion'})) {
+			push @valid_robot_candidates,$robot;
+		    }
+		}
+	    }
+	    if ($#valid_robot_candidates == 0) {
+		$valid_robot = $valid_robot_candidates[0];
+		my $statement = sprintf "UPDATE exclusion_table SET robot_exclusion = %s WHERE list_exclusion=%s AND user_exclusion=%s", $dbh->quote($valid_robot),$dbh->quote($data->{'list_exclusion'}),$dbh->quote($data->{'user_exclusion'});  
+		my $sth;
+		unless ($sth = $dbh->prepare($statement)) {
+		    &Log::do_log('err','Unable to prepare SQL statement : %s', $dbh->errstr);
+		}
+		unless ($sth->execute) {
+		    &Log::do_log('err','Unable to execute SQL statement "%s" : %s', $statement, $dbh->errstr);
+		}
+	    }else {
+		&Log::do_log('err',"Exclusion robot could not be guessed for user '%s' in list '%s'. Either this user is no longer subscribed to the list or the list appear in more than one robot (or the query to the database failed). Here is the list of robots in which this list name appears: '%s'",$data->{'list_exclusion'},$data->{'user_exclusion'},@valid_robot_candidates);
+	    }
+	}
+	## Caching all lists config subset to database
+	&Log::do_log('notice','Caching all lists config subset to database');
+	&List::_flush_list_db();
+	my $all_lists = &List::get_lists('*', { 'use_files' => 1 });
+	foreach my $list (@$all_lists) {
+	    $list->_update_list_db;
+	}
+   }
 
     return 1;
 }
@@ -755,6 +802,7 @@ sub probe_db {
 						   'info_admin' =>  'varchar(150)',
 						   'profile_admin' => "enum('privileged','normal')"},
 				 'exclusion_table' => {'list_exclusion' => 'varchar(50)',
+						       'robot_exclusion' => 'varchar(50)',
 						       'user_exclusion' => 'varchar(100)',
 						       'date_exclusion' => 'int(11)'},
 				 'netidmap_table' => {'netid_netidmap' => 'varchar(100)',
@@ -815,8 +863,19 @@ sub probe_db {
 						   },
 				 'conf_table' => {'robot_conf' => 'varchar(80)',
 						  'label_conf' => 'varchar(80)',
-						  'value_conf' => 'varchar(300)'}
-			     },
+						  'value_conf' => 'varchar(300)'},
+                  'list_table' => {'name_list'=>'varchar(100)',
+                                    'path_list'=>'varchar(100)',
+                                    'robot_list'=>'varchar(100)',
+                                    'status_list'=>"enum('open','closed','pending','error_config','family_closed')",
+                                    'creation_email_list'=>'varchar(100)',
+                                    'creation_epoch_list'=>'datetime',
+                                    'subject_list'=>'varchar(100)',
+                                    'web_archive_list'=>'tinyint(1)',
+                                    'topics_list'=>'varchar(100)',
+                                    'editors_list'=>'varchar(100)',
+                                    'owners_list'=>'varchar(100)'}
+                              },
 		     'SQLite' => {'user_table' => {'email_user' => 'text',
 						   'gecos_user' => 'text',
 						   'password_user' => 'text',
@@ -862,6 +921,7 @@ sub probe_db {
 						    'profile_admin' => 'text'},
 				  'exclusion_table' => {'list_exclusion' => 'text',
 							'user_exclusion' => 'text',
+							'robot_exclusion' => 'text',
 							'date_exclusion' => 'integer'},
 				  'netidmap_table' => {'netid_netidmap' => 'text',
 						       'serviceid_netidmap' => 'text',
@@ -921,7 +981,19 @@ sub probe_db {
 							'dkim_header_list_bulkspool' => 'varchar(500)'},
 				  'conf_table' => {'robot_conf' => 'text',
 						   'label_conf' => 'text',
-						   'value_conf' => 'text'}});
+						   'value_conf' => 'text'},
+                  'list_table' => {'name_list'=>'varchar(100)',
+                                    'path_list'=>'varchar(100)',
+                                    'robot_list'=>'varchar(100)',
+                                    'status_list'=>"enum('open','closed','pending','error_config','family_closed')",
+                                    'creation_email_list'=>'varchar(100)',
+                                    'creation_epoch_list'=>'datetime',
+                                    'subject_list'=>'varchar(100)',
+                                    'web_archive_list'=>'tinyint(1)',
+                                    'topics_list'=>'varchar(100)',
+                                    'editors_list'=>'varchar(100)',
+                                    'owners_list'=>'varchar(100)'}
+});
 
     my %not_null = ('email_user' => 1,
 		    'list_subscriber' => 1,
@@ -935,6 +1007,7 @@ sub probe_db {
 		    'date_admin' => 1,
 		    'list_exclusion' => 1,
 		    'user_exclusion' => 1,
+		    'robot_exclusion' => 1,
 		    'netid_netidmap' => 1,
 		    'serviceid_netidmap' => 1,
 		    'robot_netidmap' => 1,
@@ -949,19 +1022,30 @@ sub probe_db {
 		    'messagekey_bulkmailer' => 1,
 		    'packetid_bulkmailer' => 1,
 		    'messagekey_bulkspool' => 1,
+             'name_list'=>1,
+             'path_list'=>1,
+             'robot_list'=>1,
+             'status_list'=>1,
+             'creation_email_list'=>1,
+             'subject_list'=>1,
+             'web_archive_list'=>1,
+             'topics_list'=>1,
+             'owners_list'=>1,
+             'editors_list'=>1
 		    );
     
     my %primary = ('user_table' => ['email_user'],
 		   'subscriber_table' => ['robot_subscriber','list_subscriber','user_subscriber'],
 		   'admin_table' => ['robot_admin','list_admin','role_admin','user_admin'],
-		   'exclusion_table' => ['list_exclusion','user_exclusion'],
+		   'exclusion_table' => ['list_exclusion','user_exclusion','robot_exclusion'],
 		   'netidmap_table' => ['netid_netidmap','serviceid_netidmap','robot_netidmap'],
 		   'logs_table' => ['id_logs'],
 		   'session_table' => ['id_session'],
 		   'one_time_ticket_table' => ['ticket_one_time_ticket'],
 		   'bulkmailer_table' => ['messagekey_bulkmailer','packetid_bulkmailer'],
 		   'bulkspool_table' => ['messagekey_bulkspool'],
-		   'conf_table' => ['robot_conf','label_conf']
+		   'conf_table' => ['robot_conf','label_conf'],
+		   'list_table'=> ['name_list','robot_list']
 		   );
 
     ## List the required INDEXES
