@@ -1,5 +1,5 @@
 # admin.pm - This module includes administrative function for the lists
-# RCS Identication ; $Revision: 6981 $ ; $Date: 2011-01-18 15:56:37 +0100 (mar 18 jan 2011) $ 
+# RCS Identication ; $Revision: 7327 $ ; $Date: 2012-04-17 05:59:10 +0200 (mar 17 avr 2012) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -340,7 +340,9 @@ sub create_list_old{
     if ($list->has_include_data_sources()) {
 	&do_log('notice', "Synchronizing list members...");
 	$list->sync_include();
-    }   
+    }
+    
+    $list->save_config;
 
     return $return;
 }
@@ -441,7 +443,7 @@ sub create_list{
                 $param->{'listname'}, $family->{'name'},$robot);
       return undef;
     }
-
+    
      ## Create the list directory
      my $list_dir;
 
@@ -500,7 +502,25 @@ sub create_list{
 	print INFO $param->{'description'};
     }
     close INFO;
-    
+
+    ## Create associated files if a template was given.
+    for my $file ('message.footer','message.header','message.footer.mime','message.header.mime','info') {
+	my $template_file = &tools::get_filename('etc',{},$file.".tt2", $robot,$family);
+	if (defined $template_file) {
+	    my $file_content;
+	    my $tt_result = &tt2::parse_tt2($param, $file.".tt2", \$file_content, [$family->{'dir'}]);
+	    unless (defined $tt_result) {
+		&do_log('err', 'admin::create_list : tt2 error. List %s from family %s@%s, file %s',
+			$param->{'listname'}, $family->{'name'},$robot,$file);
+	    }
+	    unless (open FILE, '>:utf8', "$list_dir/$file") {
+		&do_log('err','Impossible to create %s/%s : %s',$list_dir,$file,$!);
+	    }
+	    print FILE $file_content;
+	    close FILE;
+	}
+    }
+
     ## Create list object
     my $list;
     unless ($list = new List ($param->{'listname'}, $robot)) {
@@ -801,9 +821,8 @@ sub rename_list{
 	 }
 	 
 	 # if subscribtion are stored in database rewrite the database
-	 if ($list->{'admin'}{'user_data_source'} =~ /^(database|include2)$/) {
-	     &List::rename_list_db ($list,$param{'new_listname'},$param{'new_robot'});
-	 }
+	 &List::rename_list_db($list, $param{'new_listname'},
+			       $param{'new_robot'});
      }
      ## Install new aliases
      $param{'listname'} = $param{'new_listname'};
@@ -835,10 +854,11 @@ sub rename_list{
 		 &do_log('err', "Unable to open '%s' spool : %s", $Conf::Conf{$spool}, $!);
 	     }
 	     
-	     foreach my $file (sort grep (!/^\.+$/,readdir(DIR))) {
+	     foreach my $file (sort readdir(DIR)) {
 		 next unless ($file =~ /^$old_listname\_/ ||
 			      $file =~ /^$old_listname\./ ||
 			      $file =~ /^$old_listname\@$robot\./ ||
+			      $file =~ /^\.$old_listname\@$robot\_/ ||
 			      $file =~ /^$old_listname\@$robot\_/ ||
 			      $file =~ /\.$old_listname$/);
 		 
@@ -851,6 +871,8 @@ sub rename_list{
 		     $newfile =~ s/^$old_listname\@$robot\./$param{'new_listname'}\@$param{'new_robot'}\./;
 		 }elsif ($file =~ /^$old_listname\@$robot\_/) {
 		     $newfile =~ s/^$old_listname\@$robot\_/$param{'new_listname'}\@$param{'new_robot'}\_/;
+		 }elsif ($file =~ /^\.$old_listname\@$robot\_/) {
+		     $newfile =~ s/^\.$old_listname\@$robot\_/\.$param{'new_listname'}\@$param{'new_robot'}\_/;
 		 }elsif ($file =~ /\.$old_listname$/) {
 		     $newfile =~ s/\.$old_listname$/\.$param{'new_listname'}/;
 		 }
@@ -1139,7 +1161,7 @@ sub install_aliases {
 	 system ("$alias_manager add $list->{'name'} $list->{'admin'}{'host'}") ;
 	 my $status = $? / 256;
 	 if ($status == 0) {
-	     &do_log('err','admin::install_aliases : Aliases installed successfully') ;
+	     &do_log('info','admin::install_aliases : Aliases installed successfully') ;
 	     $alias_installed = 1;
 	 }elsif ($status == 1) {
 	     &do_log('err','admin::install_aliases : Configuration file %s has errors', Sympa::Constants::CONFIG);
